@@ -1,31 +1,25 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 // Declarations for simple estimate of CPU time being used by a program.
 // This header is an optional part of the test harness.
 // It assumes that "harness_assert.h" has already been included.
 
-#if _WIN32 
-#if !_XBOX
+#if _WIN32
     #include <windows.h>
-#endif
 #else
     #include <sys/time.h>
     #include <sys/resource.h>
@@ -33,8 +27,8 @@
 
 //! Return time (in seconds) spent by the current process in user mode.
 /*  Returns 0 if not implemented on platform. */
-static double GetCPUUserTime() { 
-#if _XBOX || __TBB_WIN8UI_SUPPORT
+static double GetCPUUserTime() {
+#if __TBB_WIN8UI_SUPPORT
     return 0;
 #elif _WIN32
     FILETIME my_times[4];
@@ -51,7 +45,7 @@ static double GetCPUUserTime() {
     //   getrusage(RUSAGE_SELF, ...) that is used now only returns info for the calling thread;
     //   getrusage(RUSAGE_CHILDREN, ...) only counts for finished children threads;
     //   tms_utime and tms_cutime got with times(struct tms*) are equivalent to the above items;
-    //   finally, /proc/self/task/<task_id>/stat doesn't exist on older kernels 
+    //   finally, /proc/self/task/<task_id>/stat doesn't exist on older kernels
     //      and it isn't quite convenient to read it for every task_id.
 
     struct rusage resources;
@@ -87,7 +81,7 @@ static void TestCPUUserTime( int nthreads, int nactive = 1 ) {
     while( (usrtime_delta=GetCPUUserTime()-lastusrtime) < THRESHOLD ) {
         for ( int i = 0; i < 1000; ++i ) ++k; // do fake work without which user time can stall
         if ( (waittime_delta = (tbb::tick_count::now()-stamp).seconds()) > maximal_waittime ) {
-            REPORT( "Warning: %.2f sec elapsed but user mode time is still below its threshold (%g < %g)\n", 
+            REPORT( "Warning: %.2f sec elapsed but user mode time is still below its threshold (%g < %g)\n",
                     waittime_delta, usrtime_delta, THRESHOLD );
             break;
         }
@@ -96,7 +90,7 @@ static void TestCPUUserTime( int nthreads, int nactive = 1 ) {
 
     // Wait for workers to go sleep
     stamp = tbb::tick_count::now();
-    while( ((waittime_delta=(tbb::tick_count::now()-stamp).seconds()) < minimal_waittime) 
+    while( ((waittime_delta=(tbb::tick_count::now()-stamp).seconds()) < minimal_waittime)
             || ((usrtime_delta=GetCPUUserTime()-lastusrtime) < THRESHOLD) )
     {
         for ( int i = 0; i < 1000; ++i ) ++k; // do fake work without which user time can stall
