@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "tbb/tbb_config.h"
@@ -80,7 +76,7 @@ typedef void *LIBRARY_HANDLE;
 LIBRARY_HANDLE OpenLibrary(const char *name)
 {
 #if _WIN32 || _WIN64
-#if __TBB_WIN8UI_SUPPORT	
+#if __TBB_WIN8UI_SUPPORT
     TCHAR wlibrary[MAX_PATH];
     if ( MultiByteToWideChar(CP_UTF8, 0, name, -1, wlibrary, MAX_PATH) == 0 ) return false;
     return :: LoadPackagedLibrary( wlibrary, 0 );
@@ -105,16 +101,22 @@ void CloseLibrary(LIBRARY_HANDLE lib)
 
 typedef void (*FunctionAddress)();
 
+template <typename FunctionPointer>
+void GetAddress(Harness::LIBRARY_HANDLE lib, const char *name, FunctionPointer& func)
+{
+#if _WIN32 || _WIN64
+    func = (FunctionPointer)(void*)GetProcAddress(lib, name);
+#else
+    func = (FunctionPointer)dlsym(lib, name);
+#endif
+    ASSERT(func, "Can't find required symbol in dynamic library");
+}
+
 FunctionAddress GetAddress(Harness::LIBRARY_HANDLE lib, const char *name)
 {
-    union { FunctionAddress func; void *symb; } converter;
-#if _WIN32 || _WIN64
-    converter.symb = (void*)GetProcAddress(lib, name);
-#else
-    converter.symb = (void*)dlsym(lib, name);
-#endif
-    ASSERT(converter.func, "Can't find required symbol in dynamic library");
-    return converter.func;
+    FunctionAddress func;
+    GetAddress(lib, name, func);
+    return func;
 }
 
 }  // namespace Harness
