@@ -1,23 +1,22 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
+#define HARNESS_DEFAULT_MIN_THREADS 6
+#define HARNESS_DEFAULT_MAX_THREADS 8
+
 #include "tbb/concurrent_monitor.h"
 #include "tbb/atomic.h"
 #include "tbb/task_scheduler_init.h"
@@ -34,7 +33,7 @@
 #if _MSC_VER && !defined(__INTEL_COMPILER)
     // Workaround for overzealous compiler warnings
     // Suppress compiler warning about constant conditional expression
-    #pragma warning (disable: 4127)
+    // #pragma warning (disable: 4127)
 #endif
 
 using namespace tbb;
@@ -115,7 +114,7 @@ void QueuingMutex::ScopedLock::Acquire( QueuingMutex& m, size_t test_mode )
         case 0:
             mutex->waitq.wait( QueuingMutex_Until(going), QueuingMutex_Context(this) );
             break;
-#if __TBB_LAMBDAS_PRESENT
+#if __TBB_CPP11_LAMBDAS_PRESENT
         case 1:
             mutex->waitq.wait( [&](){ return going!=0ul; }, [=]() { return (uintptr_t)this; } );
             break;
@@ -216,7 +215,7 @@ retry:
         case 0:
             mutex->waitq.wait( SpinMutex_Until(mutex), SpinMutex_Context(this) );
             break;
-#if __TBB_LAMBDAS_PRESENT
+#if __TBB_CPP11_LAMBDAS_PRESENT
         case 1:
             mutex->waitq.wait( [&](){ return mutex->flag==0; }, [=]() { return (uintptr_t)this; } );
             break;
@@ -299,7 +298,7 @@ atomic<size_t> n_sleepers;
 
 #if defined(_MSC_VER) && defined(_Wp64)
     // Workaround for overzealous compiler warnings in /Wp64 mode
-    #pragma warning (disable: 4244 4267)
+    // #pragma warning (disable: 4244 4267)
 #endif
 
 struct AllButOneSleep : NoAssign {
@@ -325,7 +324,7 @@ struct AllButOneSleep : NoAssign {
                 try {
                     ++n_sleepers;
                     mon->commit_wait( thr_ctx );
-                    if( --n_sleepers>VLN ) 
+                    if( --n_sleepers>VLN )
                         break;
                 } catch( tbb::user_abort& ) {
                     // can no longer access 'mon'
