@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #define HARNESS_NO_PARSE_COMMAND_LINE 1
@@ -103,7 +99,7 @@ bool TestReallocMsize(size_t startSz) {
     ASSERT(buf, "");
     size_t realSz = scalable_msize(buf);
     ASSERT(realSz>=startSz, "scalable_msize must be not less then allocated size");
-    memset(buf, 'a', realSz-1);
+    memset(static_cast<void*>(buf), 'a', realSz-1);
     buf[realSz-1] = 0;
     char *buf1 = (char*)scalable_realloc(buf, 2*realSz);
     ASSERT(buf1, "");
@@ -126,32 +122,41 @@ void TestAlignedMsize()
     const int NUM = 4;
     char *p[NUM];
     size_t objSizes[NUM];
-    size_t allocSz[] = {8, 512, 2*1024, 4*2024, 8*1024, 0};
+    size_t allocSz[] = {4, 8, 512, 2*1024, 4*1024, 8*1024, 16*1024, 0};
+    size_t align[] = {8, 512, 2*1024, 4*1024, 8*1024, 16*1024, 0};
 
-    for (int s=0; allocSz[s]; s++) {
-        for (int i=0; i<NUM; i++)
-            p[i] = (char*)scalable_aligned_malloc(16, allocSz[s]);
+    for (int a=0; align[a]; a++)
+        for (int s=0; allocSz[s]; s++) {
+            for (int i=0; i<NUM; i++) {
+                p[i] = (char*)scalable_aligned_malloc(allocSz[s], align[a]);
+                ASSERT(is_aligned(p[i], align[a]), NULL);
+            }
 
-        for (int i=0; i<NUM; i++) {
-            objSizes[i] = scalable_msize(p[i]);
-            memset(p[i], i, objSizes[i]);
-        }
-        for (int i=0; i<NUM; i++) {
-            for (unsigned j=0; j<objSizes[i]; j++)
-                ASSERT(((char*)p[i])[j] == i, "Error: data broken\n");
-        }
+            for (int i=0; i<NUM; i++) {
+                objSizes[i] = scalable_msize(p[i]);
+                ASSERT(objSizes[i] >= allocSz[s],
+                       "allocated size must be not less than requested");
+                memset(p[i], i, objSizes[i]);
+            }
+            for (int i=0; i<NUM; i++) {
+                for (unsigned j=0; j<objSizes[i]; j++)
+                    ASSERT(((char*)p[i])[j] == i, "Error: data broken");
+            }
 
-        for (int i=0; i<NUM; i++) {
-            p[i] = (char*)scalable_aligned_realloc(p[i], allocSz[s], 16);
-            memset(p[i], i, allocSz[s]);
+            for (int i=0; i<NUM; i++) {
+                p[i] = (char*)scalable_aligned_realloc(p[i], 2*allocSz[s], align[a]);
+                ASSERT(is_aligned(p[i], align[a]), NULL);
+                memset((char*)p[i]+allocSz[s], i+1, allocSz[s]);
+            }
+            for (int i=0; i<NUM; i++) {
+                for (unsigned j=0; j<allocSz[s]; j++)
+                    ASSERT(((char*)p[i])[j] == i, "Error: data broken");
+                for (size_t j=allocSz[s]; j<2*allocSz[s]; j++)
+                    ASSERT(((char*)p[i])[j] == i+1, "Error: data broken");
+            }
+            for (int i=0; i<NUM; i++)
+                scalable_free(p[i]);
         }
-        for (int i=0; i<NUM; i++) {
-            for (unsigned j=0; j<objSizes[i]; j++)
-                ASSERT(((char*)p[i])[j] == i, "Error: data broken\n");
-        }
-        for (int i=0; i<NUM; i++)
-            scalable_free(p[i]);
-    }
 }
 
 /*--------------------------------------------------------------------*/
