@@ -1,25 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
-#pragma warning(disable: 4180) // "qualifier applied to function type has no meaning; ignored"
+// #pragma warning(disable: 4180) // "qualifier applied to function type has no meaning; ignored"
 #endif
 
 #include "tbb/parallel_for_each.h"
@@ -29,7 +25,7 @@
 #include "harness_iterator.h"
 #include <list>
 
-// Some old compilers can't deduce template paremeter type for parallel_for_each
+// Some old compilers can't deduce template parameter type for parallel_for_each
 // if the function name is passed without explicit cast to function pointer.
 typedef void (*TestFunctionType)(size_t);
 
@@ -51,7 +47,7 @@ void RunPForEachTests()
     sum = 0;
     size_t test_sum = 0;
 
-    for (size_t i =0; i < NUMBER_OF_ELEMENTS; i++) { 
+    for (size_t i =0; i < NUMBER_OF_ELEMENTS; i++) {
         test_vector[i] = i;
         test_sum += i;
     }
@@ -102,7 +98,7 @@ void TestExceptionsSupport()
     REMARK (__FUNCTION__);
     size_t test_vector[NUMBER_OF_ELEMENTS + 1];
 
-    for (size_t i = 0; i < NUMBER_OF_ELEMENTS; i++) { 
+    for (size_t i = 0; i < NUMBER_OF_ELEMENTS; i++) {
         test_vector[i] = i;
     }
 
@@ -115,7 +111,7 @@ void TestExceptionsSupport()
 }
 #endif /* TBB_USE_EXCEPTIONS */
 
-// Cancelation support test
+// Cancellation support test
 void function_to_cancel(size_t ) {
     ++g_CurExecuted;
     CancellatorTask::WaitUntilReady();
@@ -126,16 +122,16 @@ class my_worker_pforeach_task : public tbb::task
 {
     tbb::task_group_context &my_ctx;
 
-    tbb::task* execute () {
+    tbb::task* execute () __TBB_override {
         size_t test_vector[NUMBER_OF_ELEMENTS + 1];
-        for (size_t i = 0; i < NUMBER_OF_ELEMENTS; i++) { 
+        for (size_t i = 0; i < NUMBER_OF_ELEMENTS; i++) {
             test_vector[i] = i;
         }
         Iterator begin(&test_vector[0]);
         Iterator end(&test_vector[NUMBER_OF_ELEMENTS]);
 
         tbb::parallel_for_each(begin, end, (TestFunctionType)function_to_cancel);
-        
+
         return NULL;
     }
 public:
