@@ -1,29 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks.
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
+        http://www.apache.org/licenses/LICENSE-2.0
 
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "harness_defs.h"
@@ -64,13 +52,16 @@
 #else /* !TBBTEST_USE_TBB */
 
     #if defined(_MSC_VER)
-    #pragma warning(disable: 4100 4180)
+    // #pragma warning(disable: 4100 4180)
     #endif
 
     #include <ppl.h>
 
     typedef unsigned int uint_t;
 
+    // Bug in this ConcRT version results in task_group::wait() rethrowing
+    // internal cancellation exception propagated by the scheduler from the nesting
+    // task group.
     #define __TBB_SILENT_CANCELLATION_BROKEN  (_MSC_VER == 1600)
 
 #endif /* !TBBTEST_USE_TBB */
@@ -79,6 +70,7 @@
 
 #include "tbb/atomic.h"
 #include "tbb/aligned_space.h"
+#include "harness.h"
 #include "harness_concurrency_tracker.h"
 
 unsigned g_MaxConcurrency = 0;
@@ -231,12 +223,44 @@ public:
     void operator() ( uint_t idx ) const { m_pImpl->Run(idx); }
 };
 
+class RunAndWaitSyncronizationTestBody : NoAssign {
+    Harness::SpinBarrier& m_barrier;
+    tbb::atomic<bool>& m_completed;
+    tbb::task_group& m_tg;
+public:
+    RunAndWaitSyncronizationTestBody(Harness::SpinBarrier& barrier, tbb::atomic<bool>& completed, tbb::task_group& tg)
+        : m_barrier(barrier), m_completed(completed), m_tg(tg) {}
+
+    void operator()() const {
+        m_barrier.wait();
+        for (volatile int i = 0; i < 100000; ++i) {}
+        m_completed = true;
+    }
+
+    void operator()(int id) const {
+        if (id == 0) {
+            m_tg.run_and_wait(*this);
+        } else {
+            m_barrier.wait();
+            m_tg.wait();
+            ASSERT(m_completed, "A concurrent waiter has left the wait method earlier than work has finished");
+        }
+    }
+};
+
 void TestParallelSpawn () {
     NativeParallelFor( g_MaxConcurrency, SharedGroupBody(g_MaxConcurrency) );
 }
 
 void TestParallelWait () {
     NativeParallelFor( g_MaxConcurrency, SharedGroupBody(g_MaxConcurrency, ParallelWait) );
+
+    Harness::SpinBarrier barrier(g_MaxConcurrency);
+    tbb::atomic<bool> completed;
+    completed = false;
+    tbb::task_group tg;
+    RunAndWaitSyncronizationTestBody b(barrier, completed, tg);
+    NativeParallelFor( g_MaxConcurrency, b );
 }
 
 // Tests non-stack-bound task group (the group that is allocated by one thread and destroyed by the other)
@@ -476,7 +500,7 @@ void TestTaskHandle2 () {
     FIB_TEST_EPILOGUE(g_Sum);
 }
 
-#if __TBB_LAMBDAS_PRESENT
+#if __TBB_CPP11_LAMBDAS_PRESENT
 //------------------------------------------------------------------------
 // Test for a mixed tree of task groups.
 // The chores are specified as lambdas
@@ -510,7 +534,7 @@ void TestFibWithMakeTask () {
     rg.run_and_wait( h2 );
     ASSERT( sum == 2 * F, NULL );
 }
-#endif /* __TBB_LAMBDAS_PRESENT */
+#endif /* __TBB_CPP11_LAMBDAS_PRESENT */
 
 
 //------------------------------------------------------------------------
@@ -523,7 +547,7 @@ class test_exception : public std::exception
 public:
     test_exception ( const char* descr ) : m_strDescription(descr) {}
 
-    const char* what() const throw() { return m_strDescription; }
+    const char* what() const throw() __TBB_override { return m_strDescription; }
 };
 
 #if TBB_USE_CAPTURED_EXCEPTION
@@ -770,7 +794,7 @@ void TestStructuredCancellation1 () {
 
 #if TBB_USE_EXCEPTIONS
 #if defined(_MSC_VER)
-    #pragma warning (disable: 4127)
+    // #pragma warning (disable: 4127)
 #endif
 
 template<bool Throw>
@@ -814,6 +838,128 @@ void TestStructuredWait () {
     sg.wait();
 }
 
+struct TestFunctor {
+    void operator()() { ASSERT( false, "Non-const operator called" ); }
+    void operator()() const { /* library requires this overload only */ }
+};
+
+void TestConstantFunctorRequirement() {
+    tbb::task_group g;
+    TestFunctor tf;
+    g.run( tf ); g.wait();
+    g.run_and_wait( tf );
+}
+//------------------------------------------------------------------------
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+namespace TestMoveSemanticsNS {
+    struct TestFunctor {
+        void operator()() const {};
+    };
+
+    struct MoveOnlyFunctor : MoveOnly, TestFunctor {
+        MoveOnlyFunctor() : MoveOnly() {};
+        MoveOnlyFunctor(MoveOnlyFunctor&& other) : MoveOnly(std::move(other)) {};
+    };
+
+    struct MovePreferableFunctor : Movable, TestFunctor {
+        MovePreferableFunctor() : Movable() {};
+        MovePreferableFunctor(MovePreferableFunctor&& other) : Movable(std::move(other)) {};
+        MovePreferableFunctor(const MovePreferableFunctor& other) : Movable(other) {};
+    };
+
+    struct NoMoveNoCopyFunctor : NoCopy, TestFunctor {
+        NoMoveNoCopyFunctor() : NoCopy() {};
+        // mv ctor is not allowed as cp ctor from parent NoCopy
+    private:
+        NoMoveNoCopyFunctor(NoMoveNoCopyFunctor&&);
+    };
+
+    void TestFunctorsWithinTaskHandles() {
+        // working with task_handle rvalues is not supported in task_group
+
+        tbb::task_group tg;
+        MovePreferableFunctor mpf;
+        typedef tbb::task_handle<MoveOnlyFunctor> th_mv_only_type;
+        typedef tbb::task_handle<MovePreferableFunctor> th_mv_pref_type;
+
+        th_mv_only_type th_mv_only = th_mv_only_type(MoveOnlyFunctor());
+        tg.run_and_wait(th_mv_only);
+
+        th_mv_only_type th_mv_only1 = th_mv_only_type(MoveOnlyFunctor());
+        tg.run(th_mv_only1);
+        tg.wait();
+
+        th_mv_pref_type th_mv_pref = th_mv_pref_type(mpf);
+        tg.run_and_wait(th_mv_pref);
+        ASSERT(mpf.alive, "object was moved when was passed by lval");
+        mpf.Reset();
+
+        th_mv_pref_type th_mv_pref1 = th_mv_pref_type(std::move(mpf));
+        tg.run_and_wait(th_mv_pref1);
+        ASSERT(!mpf.alive, "object was copied when was passed by rval");
+        mpf.Reset();
+
+        th_mv_pref_type th_mv_pref2 = th_mv_pref_type(mpf);
+        tg.run(th_mv_pref2);
+        tg.wait();
+        ASSERT(mpf.alive, "object was moved when was passed by lval");
+        mpf.Reset();
+
+        th_mv_pref_type th_mv_pref3 = th_mv_pref_type(std::move(mpf));
+        tg.run(th_mv_pref3);
+        tg.wait();
+        ASSERT(!mpf.alive, "object was copied when was passed by rval");
+        mpf.Reset();
+    }
+
+    void TestBareFunctors() {
+        tbb::task_group tg;
+        MovePreferableFunctor mpf;
+        // run_and_wait() doesn't have any copies or moves of arguments inside the impl
+        tg.run_and_wait( NoMoveNoCopyFunctor() );
+
+        tg.run( MoveOnlyFunctor() );
+        tg.wait();
+
+        tg.run( mpf );
+        tg.wait();
+        ASSERT(mpf.alive, "object was moved when was passed by lval");
+        mpf.Reset();
+
+        tg.run( std::move(mpf) );
+        tg.wait();
+        ASSERT(!mpf.alive, "object was copied when was passed by rval");
+        mpf.Reset();
+    }
+
+    void TestMakeTask() {
+        MovePreferableFunctor mpf;
+
+        tbb::make_task( MoveOnly() );
+
+        tbb::make_task( mpf );
+        ASSERT(mpf.alive, "object was moved when was passed by lval");
+        mpf.Reset();
+
+        tbb::make_task( std::move(mpf) );
+        ASSERT(!mpf.alive, "object was copied when was passed by rval");
+        mpf.Reset();
+    }
+}
+#endif /* __TBB_CPP11_RVALUE_REF_PRESENT */
+
+void TestMoveSemantics() {
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    TestMoveSemanticsNS::TestBareFunctors();
+    TestMoveSemanticsNS::TestFunctorsWithinTaskHandles();
+    TestMoveSemanticsNS::TestMakeTask();
+#else
+    REPORT("Known issue: move support tests are skipped.\n");
+#endif
+}
+//------------------------------------------------------------------------
+
+
 int TestMain () {
     REMARK ("Testing %s task_group functionality\n", TBBTEST_USE_TBB ? "TBB" : "PPL");
     for( int p=MinThread; p<=MaxThread; ++p ) {
@@ -838,7 +984,7 @@ int TestMain () {
         TestTaskHandle();
         TestTaskHandle2<Concurrency::task_group>();
         TestTaskHandle2<Concurrency::structured_task_group>();
-#if __TBB_LAMBDAS_PRESENT
+#if __TBB_CPP11_LAMBDAS_PRESENT
         TestFibWithLambdas();
         TestFibWithMakeTask();
 #endif
@@ -849,7 +995,7 @@ int TestMain () {
         TestEh2();
         TestStructuredWait();
         TestStructuredCancellation2<true>();
-#if !__TBB_THROW_FROM_DTOR_BROKEN
+#if !(__TBB_THROW_FROM_DTOR_BROKEN || __TBB_STD_UNCAUGHT_EXCEPTION_BROKEN)
         TestStructuredCancellation2<false>();
 #else
         REPORT("Known issue: TestStructuredCancellation2<false>() is skipped.\n");
@@ -859,9 +1005,11 @@ int TestMain () {
         s->Release();
 #endif
     }
+    TestConstantFunctorRequirement();
 #if __TBB_THROW_ACROSS_MODULE_BOUNDARY_BROKEN
     REPORT("Known issue: exception handling tests are skipped.\n");
 #endif
+    TestMoveSemantics();
     return Harness::Done;
 }
 
