@@ -1,32 +1,28 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
-/*  The test uses "single produces multiple consumers" (SPMC )pattern to check 
-    if the memory of the tasks stolen by consumer threads is returned to the 
+/*  The test uses "single produces multiple consumers" (SPMC )pattern to check
+    if the memory of the tasks stolen by consumer threads is returned to the
     producer thread and is reused.
 
     The test consists of a series of iterations, which execute a task tree.
     the test fails is the memory consumption is not stabilized during some
     number of iterations.
 
-    After the memory consumption stabilized the memory state is perturbed by 
+    After the memory consumption stabilized the memory state is perturbed by
     switching producer thread, and the check is repeated.
 */
 
@@ -68,7 +64,7 @@ using namespace tbb::internal;
 
 class ChangeProducer: public tbb::task {
 public:
-    /*override*/ tbb::task* execute() {
+    tbb::task* execute() __TBB_override {
         if( is_stolen_task() ) {
             Producer = internal::governor::local_scheduler();
         }
@@ -83,7 +79,7 @@ public:
     TaskGenerator(int child_count, int d) : my_child_count(child_count), my_depth(d) {
         ASSERT(my_child_count>1, "The TaskGenerator should produce at least two children");
     }
-    /*override*/ tbb::task* execute() {
+    tbb::task* execute() __TBB_override {
         if( my_depth>0 ) {
             int child_count = my_child_count;
             scheduler* my_sched = internal::governor::local_scheduler();
@@ -120,12 +116,12 @@ public:
 #include "harness_memory.h"
 #if _MSC_VER==1500 && !defined(__INTEL_COMPILER)
     // VS2008/VC9 seems to have an issue
-    #pragma warning( push )
-    #pragma warning( disable: 4985 )
+    // #pragma warning( push )
+    // #pragma warning( disable: 4985 )
 #endif
 #include <math.h>
 #if _MSC_VER==1500 && !defined(__INTEL_COMPILER)
-    #pragma warning( pop )
+    // #pragma warning( pop )
 #endif
 
 void RunTaskGenerators( bool switchProducer = false, bool checkProducer = false ) {
@@ -158,7 +154,7 @@ class TaskList: public tbb::task {
     const int my_num_childs;
 public:
     TaskList(const int num_childs) : my_num_childs(num_childs) {}
-    tbb::task* execute() {
+    tbb::task* execute() __TBB_override {
         tbb::task_list list;
         for (int i=0; i<my_num_childs; ++i)
         {
@@ -227,7 +223,7 @@ void TestTaskReclamation() {
     int     last_error_iteration = 0,
             producer_switch_iteration = 0,
             producer_switches = 0;
-    bool    switchProducer = false, 
+    bool    switchProducer = false,
             checkProducer = false;
     for( int i=0; i < MaxIterations; ++i ) {
         // These iterations check for excessive memory use and unreasonable task count
