@@ -1,28 +1,24 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "tbb/tbb_config.h"
 #include "harness.h"
 
 #if __TBB_GCC_STRICT_ALIASING_BROKEN
-//    #pragma GCC diagnostic ignored "-Wstrict-aliasing"
+    // #pragma GCC diagnostic ignored "-Wstrict-aliasing"
 #endif
 
 #if __TBB_TASK_GROUP_CONTEXT
@@ -46,7 +42,7 @@ int BaseDepth = 0;
 const int DesiredNumThreads = 12;
 
 const int NumTests = 8;
-const int TestRepeats = 4;
+int TestSwitchBetweenMastersRepeats = 4;
 
 int g_NumMasters = 0;
 volatile intptr_t *g_LeavesExecuted = NULL;
@@ -66,8 +62,8 @@ namespace tbb {
 }
 #endif /* __TBB_TASK_PRIORITY */
 
-tbb::priority_t Low = tbb::priority_normal,
-                High = tbb::priority_high;
+tbb::priority_t Low,
+                High;
 int PreemptionActivatorId = 1;
 
 enum Options {
@@ -91,7 +87,7 @@ void PrepareGlobals ( int numMasters ) {
         g_LeavesExecuted = new intptr_t[numMasters];
     g_CurConfig = 0;
     memset( const_cast<intptr_t*>(g_LeavesExecuted), 0, sizeof(intptr_t) * numMasters );
-    memset( g_TestFailures, 0, sizeof(int) * NumTests );
+    memset( static_cast<void*>(g_TestFailures), 0, sizeof(int) * NumTests );
 }
 
 void ClearGlobals () {
@@ -106,7 +102,7 @@ class LeafTask : public tbb::task {
     int m_tid;
     uintptr_t m_opts;
 
-    tbb::task* execute () {
+    tbb::task* execute () __TBB_override {
         volatile int anchor = 0;
         for ( int i = 0; i < NumIterations; ++i )
             anchor += i;
@@ -151,7 +147,7 @@ protected:
         parent_node->set_ref_count(NumLeafTasks + 1);
         --m_depth;
         for ( int i = 0; i < NumLeafTasks; ++i ) {
-            task *t = m_depth ? (task*) new(parent_node->allocate_child()) NodeType(m_tid, m_depth, m_opts, m_root) 
+            task *t = m_depth ? (task*) new(parent_node->allocate_child()) NodeType(m_tid, m_depth, m_opts, m_root)
                               : (task*) new(parent_node->allocate_child()) LeafTask(m_tid, m_opts);
             task::spawn(*t);
         }
@@ -164,7 +160,7 @@ public:
 };
 
 class NestedGroupNodeTask : public NodeTask<NestedGroupNodeTask> {
-    task* execute () {
+    task* execute () __TBB_override {
         tbb::task_group_context ctx; // Use bound context
         tbb::empty_task &r = *new( task::allocate_root(ctx) ) tbb::empty_task;
         SpawnChildren(&r);
@@ -179,7 +175,7 @@ public:
 };
 
 class BlockingNodeTask : public NodeTask<BlockingNodeTask> {
-    task* execute () {
+    task* execute () __TBB_override {
         SpawnChildren(this);
         wait_for_all();
         return NULL;
@@ -190,7 +186,7 @@ public:
 };
 
 class NonblockingNodeTask : public NodeTask<NonblockingNodeTask> {
-    task* execute () {
+    task* execute () __TBB_override {
         if ( m_depth < 0 )
             return NULL; // I'm just a continuation now
         recycle_as_safe_continuation();
@@ -298,9 +294,10 @@ void TestPrioritySwitchBetweenTwoMasters () {
         REPORT_ONCE( "Known issue: TestPrioritySwitchBetweenTwoMasters is skipped for big number of threads\n" );
         return;
     }
+    tbb::task_scheduler_init init; // keeps the market alive to reduce the amount of TBB warnings
     REMARK( "Stress tests: %s / %s \n", Low == tbb::priority_low ? "Low" : "Normal", High == tbb::priority_normal ? "Normal" : "High" );
     PrepareGlobals( 2 );
-    for ( int i = 0; i < TestRepeats; ++i ) {
+    for ( int i = 0; i < TestSwitchBetweenMastersRepeats; ++i ) {
         for ( BaseDepth = MinBaseDepth; BaseDepth <= MaxBaseDepth; ++BaseDepth ) {
             RunPrioritySwitchBetweenTwoMasters<BlockingNodeTask>( 0, NoPriorities );
             RunPrioritySwitchBetweenTwoMasters<BlockingNodeTask>( 1, TestPreemption );
@@ -315,7 +312,7 @@ void TestPrioritySwitchBetweenTwoMasters () {
         }
     }
 #if __TBB_TASK_PRIORITY
-    const int NumRuns = TestRepeats * (MaxBaseDepth - MinBaseDepth + 1);
+    const int NumRuns = TestSwitchBetweenMastersRepeats * (MaxBaseDepth - MinBaseDepth + 1);
     for ( int i = 0; i < NumTests; ++i ) {
         if ( g_TestFailures[i] )
             REMARK( "Test %d: %d failures in %d runs\n", i, g_TestFailures[i], NumRuns );
@@ -331,7 +328,7 @@ void TestPrioritySwitchBetweenTwoMasters () {
 }
 
 class SingleChildRootTask : public tbb::task {
-    tbb::task* execute () {
+    tbb::task* execute () __TBB_override {
         set_ref_count(2);
         spawn ( *new(allocate_child()) tbb::empty_task );
         wait_for_all();
@@ -370,6 +367,9 @@ void EmulateWork( int ) {
 
 class PeriodicActivitiesBody {
 public:
+    static const int parallelIters[2];
+    static const int seqIters[2];
+    static int mode;
     void operator() ( int id ) const {
         tbb::task_group_context ctx;
 #if __TBB_TASK_PRIORITY
@@ -377,13 +377,17 @@ public:
 #else /* !__TBB_TASK_PRIORITY */
         (void)id;
 #endif /* !__TBB_TASK_PRIORITY */
-        for ( int i = 0; i < 5; ++i ) {
+        for ( int i = 0; i < seqIters[mode]; ++i ) {
             tbb::task_scheduler_init init;
-            tbb::parallel_for( 1, 10000, &EmulateWork, ctx );
+            tbb::parallel_for( 1, parallelIters[mode], &EmulateWork, ctx );
         }
     }
 };
 
+const int PeriodicActivitiesBody::parallelIters[] = {10000, 100};
+const int PeriodicActivitiesBody::seqIters[] = {5, 2};
+int PeriodicActivitiesBody::mode = 0;
+
 void TestPeriodicConcurrentActivities () {
     REMARK( "TestPeriodicConcurrentActivities: %s / %s \n", Low == tbb::priority_low ? "Low" : "Normal", High == tbb::priority_normal ? "Normal" : "High" );
     NativeParallelFor ( 2, PeriodicActivitiesBody() );
@@ -394,6 +398,7 @@ void TestPeriodicConcurrentActivities () {
 void TestPriorityAssertions () {
 #if TRY_BAD_EXPR_ENABLED && __TBB_TASK_PRIORITY
     REMARK( "TestPriorityAssertions\n" );
+    tbb::task_scheduler_init init; // to avoid autoinit that'd affect subsequent tests
     tbb::priority_t bad_low_priority = tbb::priority_t( tbb::priority_low - 1 ),
                     bad_high_priority = tbb::priority_t( tbb::priority_high + 1 );
     tbb::task_group_context ctx;
@@ -411,11 +416,16 @@ void TestPriorityAssertions () {
 
 tbb::atomic<tbb::priority_t> g_order;
 tbb::atomic<bool> g_order_established;
+tbb::atomic<int> g_num_tasks;
+tbb::atomic<bool> g_all_tasks_enqueued;
+int g_failures;
 class OrderedTask : public tbb::task {
     tbb::priority_t my_priority;
 public:
-    OrderedTask(tbb::priority_t p) : my_priority(p) {}
-    tbb::task* execute() {
+    OrderedTask(tbb::priority_t p) : my_priority(p) {
+        ++g_num_tasks;
+    }
+    tbb::task* execute() __TBB_override {
         tbb::priority_t prev = g_order.fetch_and_store(my_priority);
         if( my_priority != prev) {
             REMARK("prev:%s --> new:%s\n", PriorityName(prev), PriorityName(my_priority));
@@ -426,12 +436,18 @@ public:
                     g_order_established = true;
                 else ASSERT(my_priority == tbb::priority_normal && prev == tbb::priority_low, NULL);
             } else { //transition path allowed high->normal->low
-                if(prev == tbb::priority_high) ASSERT( my_priority == tbb::priority_normal, "previous priority is high - bad order");
-                else if(prev == tbb::priority_normal) ASSERT( my_priority == tbb::priority_low, "previous priority is normal - bad order");
-                else ASSERT(!g_order_established, "transition from low priority but not during initialization");
+                bool fail = prev==tbb::priority_high && my_priority!=tbb::priority_normal; // previous priority is high - bad order
+                fail |= prev==tbb::priority_normal && my_priority!=tbb::priority_low; // previous priority is normal - bad order
+                fail |= prev==tbb::priority_low; // transition from low priority but not during initialization
+                if ( fail ) {
+                    if ( g_all_tasks_enqueued )
+                        REPORT_ONCE( "ERROR: Bad order: prev = %s, my_priority = %s\n", PriorityName( prev ), PriorityName( my_priority ) );
+                    ++g_failures;
+                }
             }
         }
         EmulateWork(0);
+        --g_num_tasks;
         return NULL;
     }
     static void start(int i) {
@@ -447,10 +463,21 @@ void TestEnqueueOrder () {
     tbb::task_scheduler_init init(1); // to simplify transition checks only one extra worker for enqueue
     g_order = tbb::priority_low;
     g_order_established = false;
+    g_all_tasks_enqueued = false;
+    g_failures = 0;
     for( int i = 0; i < 1000; i++)
         OrderedTask::start(i);
-    while( g_order == tbb::priority_low ) __TBB_Yield();
-    while( g_order != tbb::priority_low ) __TBB_Yield();
+    if ( int curr_num_tasks = g_num_tasks ) {
+        // Sync with worker not to set g_all_tasks_enqueued too early.
+        while ( curr_num_tasks == g_num_tasks ) __TBB_Yield();
+    }
+    g_all_tasks_enqueued = true;
+    while( g_order == tbb::priority_low && g_num_tasks>0 ) __TBB_Yield();
+    while( g_order != tbb::priority_low && g_num_tasks>0 ) __TBB_Yield();
+    // We cannot differentiate if this misbehavior is caused by the test or by the implementation.
+    // Howerever, we do not promise mandatory priorities so we can state that the misbehavior in less
+    // than 1% cases is our best effort.
+    ASSERT( g_failures < 5, "Too many failures" );
 }
 
 namespace test_propagation {
@@ -469,7 +496,7 @@ class TestSetPriorityTask : public tbb::task {
     const int m_tree, m_i;
 public:
     TestSetPriorityTask(int t, int i) : m_tree(t), m_i(i) {}
-    tbb::task* execute() {
+    tbb::task* execute() __TBB_override {
         if( !m_i ) { // the first task creates two trees
             g_default_ctx = group();
             for( int i = 0; i <= 1; ++i ) {
@@ -497,6 +524,11 @@ public:
 void TestSetPriority() {
     REMARK("Testing set_priority() with existing forest\n");
     const int workers = last*2+1; // +1 is worker thread executing the first task
+    const int max_workers = 4*tbb::task_scheduler_init::default_num_threads();
+    if ( workers+1 > max_workers ) {
+        REPORT( "Known issue: TestSetPriority requires %d threads but due to 4P hard limit the maximum number of threads is %d\n", workers+1, max_workers );
+        return;
+    }
     tbb::task_scheduler_init init(workers+1); // +1 is master thread
     g_barrier = workers;
     is_finished = false;
@@ -511,7 +543,7 @@ void TestSetPriority() {
     for (int t = 0; t < 2; ++t)
         for (int i = first; i <= last; ++i) {
             REMARK("\r                    \rTask %i... ", i);
-            ASSERT(g_trees[t][i]->priority() == expected_priority[t][i]? tbb::priority_high : tbb::priority_normal, NULL);
+            ASSERT(g_trees[t][i]->priority() == (expected_priority[t][i]? tbb::priority_high : tbb::priority_normal), NULL);
             REMARK("OK");
         }
     REMARK("\r                    \r");
@@ -538,20 +570,50 @@ void TestSetPriority() {
             delete g_trees[t][i];
 }
 }//namespace test_propagation
+
+struct OuterParFor {
+    void operator()(int) const {
+        tbb::affinity_partitioner ap;
+        tbb::task_group_context ctx;
+        ctx.set_priority(tbb::priority_high);
+        tbb::parallel_for(0, 100, Harness::DummyBody(1000), ap, ctx);
+    }
+};
+
+// Test priorities with affinity tasks.
+void TestAffinityTasks() {
+    REMARK("Test priorities with affinity tasks\n");
+    tbb::task_scheduler_init init;
+    tbb::affinity_partitioner ap;
+    for (int i = 0; i < 10; ++i)
+        tbb::parallel_for(0, 100, OuterParFor(), ap);
+}
+
+namespace regression {
+// This is a regression test for a bug with task_group_context used from a thread that created its local scheduler but not the implicit arena
+class TestTGContext {
+public:
+    void operator() (int) const {
+        tbb::task_group_context ctx;
+        ctx.cancel_group_execution();   // initializes the local weak scheduler on the thread
+        ctx.set_priority(tbb::priority_high);
+    }
+};
+
+void TestTGContextOnNewThread() {
+    REMARK("Testing a regression for a bug with task_group_context\n");
+    TestTGContext body;
+    NativeParallelFor(1, body);
+}
+}//namespace regression_priorities
 #endif /* __TBB_TASK_PRIORITY */
 
 #if !__TBB_TEST_SKIP_AFFINITY
 #include "harness_concurrency.h"
 #endif
 
-int TestMain () {
-#if !__TBB_TEST_SKIP_AFFINITY
-    Harness::LimitNumberOfThreads( DesiredNumThreads );
-#endif
-#if !__TBB_TASK_PRIORITY
-    REMARK( "Priorities disabled: Running as just yet another task scheduler test\n" );
-#else
-    test_propagation::TestSetPriority(); // TODO: move down when bug 1996 is fixed
+int RunTests () {
+#if __TBB_TASK_PRIORITY
     TestEnqueueOrder();
 #endif /* __TBB_TASK_PRIORITY */
     TestPriorityAssertions();
@@ -561,6 +623,8 @@ int TestMain () {
     REMARK( "The number of threads: %d\n", P );
     if ( P < 3 )
         return Harness::Skipped;
+    Low = tbb::priority_normal;
+    High = tbb::priority_high;
     TestPeriodicConcurrentActivities();
     TestPrioritySwitchBetweenTwoMasters();
     Low = tbb::priority_low;
@@ -573,9 +637,31 @@ int TestMain () {
     TestPrioritySwitchBetweenTwoMasters();
     PreemptionActivatorId = 1;
     TestPrioritySwitchBetweenTwoMasters();
+    TestAffinityTasks();
+    regression::TestTGContextOnNewThread();
+
     return Harness::Done;
 }
 
+#include "tbb/global_control.h"
+
+int TestMain () {
+#if !__TBB_TEST_SKIP_AFFINITY
+    Harness::LimitNumberOfThreads( DesiredNumThreads );
+#endif
+#if !__TBB_TASK_PRIORITY
+    REMARK( "Priorities disabled: Running as just yet another task scheduler test\n" );
+#else
+    test_propagation::TestSetPriority(); // TODO: move down when bug 1996 is fixed
+#endif /* __TBB_TASK_PRIORITY */
+
+    RunTests();
+    tbb::global_control c(tbb::global_control::max_allowed_parallelism, 1);
+    PeriodicActivitiesBody::mode = 1;
+    TestSwitchBetweenMastersRepeats = 1;
+    return RunTests();
+}
+
 #else /* !__TBB_TASK_GROUP_CONTEXT */
 
 int TestMain () {
